/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import java.util.Arrays;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.patrikdufresne.ilp.Linear;
import com.patrikdufresne.ilp.LinearProblem;
import com.patrikdufresne.ilp.util.Linears;

/**
 * Minimize the number of employee assigned.
 * 
 * @author Patrik Dufresne
 */
@SuppressWarnings("unqualified-field-access")
public class MinEmployeeAssigned extends AbstractConstraintRunnable {

    private static final transient Logger LOGGER = LoggerFactory.getLogger(MinEmployeeAssigned.class);

    /**
     * The constraint's name.
     * <pre>
     * SUM from{i} to{n} xi <= %f
     * </pre>
     */
    private static final String CONSTRAINT_NAME = "MinEmployeeAssigned_%f"; //$NON-NLS-1$

    /**
     * The linear being minimized.
     */
    private Linear linear;

    /**
     * If sets, define the best known solution according to the snapshot.
     */
    private Double snapshotUpperBound;

    /**
     * Public constructor.
     * 
     * @param problem
     *            reference to ProblemBuilder
     */
    public MinEmployeeAssigned(ProblemBuilder problem) {
        super(problem);
    }

    @Override
    protected void feasible() {
        // Get the objective value
        Double value = Double.valueOf(pb.lp.getObjectiveValue().doubleValue());
        fixMaximumNumberOfEmployee(value);
    }

    /**
     * Create a redefine the constraint to sets the maximum number of employees.
     * 
     * @param value
     *            the maximum number of employee.
     */
    private void fixMaximumNumberOfEmployee(Double value) {
        // Create or redefine the constraint upper bound.
        String name = String.format(CONSTRAINT_NAME, value);
        if (this.constraint != null) {
            this.constraint.dispose();
        }
        this.constraint = pb.lp.addConstraint(name, linear, null, value);
        LOGGER.info("Minimum number of employees: {}", value); //$NON-NLS-1$
        this.feasible = true;
    }

    @Override
    protected boolean setUp() {
        LOGGER.debug("Minimize number of employee."); //$NON-NLS-1$
        // Create the following linear function : SUM xi
        Double[] coefs = new Double[pb.xi.length];
        Arrays.fill(coefs, ONE);
        this.linear = pb.lp.createLinear(coefs, pb.xi);
        // TODO Use the snapshot value to restrict the feasibility and force
        // the solver to improve the solution.
        // To reduce the computing time, sets a maximum using a constraint.
        if (pb.snapshot != null) {
            this.snapshotUpperBound = Linears.compute(linear, pb.snapshot);
            if (ZERO.equals(this.snapshotUpperBound)) {
                this.snapshotUpperBound = null;
                this.feasible = false;
                return false;
            }
            // Create a constraint to force the solver to improve the
            // current solution.
            String name = String.format(CONSTRAINT_NAME, this.snapshotUpperBound);
            this.constraint = pb.lp.addConstraint(name, linear, null, this.snapshotUpperBound);
        }
        // Set the linear as an objective.
        pb.lp.setObjectiveLinear(this.linear);
        pb.lp.setObjectiveDirection(LinearProblem.MINIMIZE);
        return true;
    }

    @Override
    protected void tearDown() {
        // Remove objective
        pb.lp.setObjectiveLinear(null);
        this.snapshotUpperBound = null;
        this.linear = null;
    }

    @Override
    protected void unfeasible() throws GeneratePlanifException {
        if (this.snapshotUpperBound != null) {
            fixMaximumNumberOfEmployee(this.snapshotUpperBound);
        } else {
            // It's a maximization problem, it' must be feasible.
            throwProblemUnFeasible();
        }
    }
}